from scapy.layers.inet import IP
from scapy.utils import chexdump


def _diff(first, second):
    if not isinstance(first, str):
        first = get_hex(first)
    if isinstance(first, str):
        first = first.split()

    if not isinstance(second, str):
        second = get_hex(second)
    if isinstance(second, str):
        second = second.split()

    if len(first) != len(second):
        print("WARN:: Frame len is not the same")

    first_row = []
    second_row = []
    for e, _ in enumerate(first):
        if first[e] != second[e]:
            first_row.append(first[e])
            second_row.append(second[e])
            continue
        first_row.append("__")
        second_row.append("__")
    status = first_row == second_row
    return first_row, second_row, status


def get_hex(frame):
    return ' '.join([x.replace("0x", "").replace(",", "") for x in chexdump(frame, dump=True).split()])


def show_hex(frame):
    print(get_hex(frame))


def show_diff(first, second, extend=False):
    first_row, second_row, status = _diff(first, second)

    print(' '.join(first_row))
    print(' '.join(second_row))

    if extend:
        more_info = []
        for types in first.class_fieldtype.items():
            for el in types[1].items():
                more_info.append((el[0], el[1].sz))
        print(more_info)

    if status:
        print("Ok")
    else:
        print("Not equal at {} point/s".format(len([x for x in first_row if x != "__"])))


def get_diff(first, second):
    _, _, status = _diff(first, second)
    return status


def table(first, second):
    f, s, status = _diff(first, second)
    show_diff(first, second)
    f_details = first.show(dump=True).split("\n")
    s_details = second.show(dump=True).split("\n")

    for r in range(len(f_details)):
        print("{} {:>40}".format(f_details[r], s_details[r]))

    return status



from unittest import TestCase

from scapy.layers.l2 import Ether

from scapy_helper.main import get_hex, _diff, show_hex, show_diff


class Test_Scapy_Helper(TestCase):
    def setUp(self) -> None:
        self.ether = Ether()

    def test_get_hex(self):
        assert get_hex(self.ether) == "ff ff ff ff ff ff e0 d5 5e e6 6c 8e 90 00"

    def test_show_diff(self):
        ether_wrong = "ff ff fc ff ff fa e0 d5 5e e6 6c 8e 90 00"
        first_row, second_row, status = _diff(self.ether, ether_wrong)
        assert first_row == ['__', '__', 'ff', '__', '__', 'ff', '__', '__', '__', '__', '__', '__', '__', '__']
        assert second_row == ['__', '__', 'fc', '__', '__', 'fa', '__', '__', '__', '__', '__', '__', '__', '__']
        assert not status

    def test_show_more_diff(self):
        show_diff(self.ether, "ff ff ff ff ff ff e0 d5 5e e6 6c 8e 90 00")



from scapy_helper.main import get_hex, show_diff, show_hex, _diff


class Compare:
    def __init__(self, first, second):
        self.first = first
        self.second = second

    def hex(self):
        v = get_hex(self.first)
        show_hex(v)
        return v
    
    def diff(self):
        f, s, status = _diff(self.first, self.second)
        show_diff(self.first, self.second)
        f_details = self.first.show(dump=True).split("\n")
        s_details = self.second.show(dump=True).split("\n")

        for r in range(len(f_details)):
            print("{} {:>40}".format(f_details[r], s_details[r]))

        return status



import unittest

from scapy_helper.compare import Compare

from scapy.layers.inet import IP, UDP
from scapy.layers.l2 import Ether
from scapy.utils import hexstr


class TestCompare(unittest.TestCase):
    def setUp(self):
        self.ether = Ether() / IP(src='192.168.1.1', dst='192.168.1.20', ttl=15) / UDP()
        self.c_ether = Ether() / IP(src='192.168.1.2', dst='192.168.1.20', ttl=20) / UDP()

    def test_one(self):
        compare = Compare(self.ether, self.c_ether)
        compare.diff()



from setuptools import setup, find_packages

setup(
    name="scapy_helper",
    description="Several features that should help you use Scapy",
    author="Nex Sabre",
    author_email="nexsabre@protonmail.com",
    version="0.1.2",
    url="https://github.com/NexSabre/scapy_helper",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=["scapy"]
)
